@@ -4,8 +4,8 @@
 #' @title Estimation of the `COTAN` model's parameters
 #'
 #' @description These functions are used to estimate the `COTAN` model's
-#'   parameters. That is the average count for each gene (lambda) the average
-#'   count for each cell (nu) and the dispersion parameter for each gene to
+#'   parameters. That is the average count for each gene (`lambda`) the average
+#'   count for each cell (`nu`) and the `dispersion` parameter for each gene to
 #'   match the probability of zero.
 #'
 #'   The estimator methods are named `Linear` if they can be calculated as a
@@ -15,11 +15,14 @@
 #' @name ParametersEstimations
 NULL
 
+
+### ------ estimateLambdaLinear -----
+
 #'
 #' @aliases estimateLambdaLinear
 #'
-#' @details `estimateLambdaLinear()` does a linear estimation of lambda (genes'
-#'   counts averages)
+#' @details `estimateLambdaLinear()` does a linear estimation of `lambda`
+#'   (genes' counts averages)
 #'
 #' @param objCOTAN a `COTAN` object
 #'
@@ -45,28 +48,18 @@ setMethod(
   function(objCOTAN) {
     lambda <- rowMeans(getRawData(objCOTAN), dims = 1L)
 
-    if (TRUE) {
-      oldLambda <- getMetadataGenes(objCOTAN)[["lambda"]]
-      if (!identical(lambda, oldLambda)) {
-        # flag the coex slots are out of sync (if any)!
-        objCOTAN@metaDataset <- updateMetaInfo(objCOTAN@metaDataset,
-                                               datasetTags()[["gsync"]], FALSE)
-        objCOTAN@metaDataset <- updateMetaInfo(objCOTAN@metaDataset,
-                                               datasetTags()[["csync"]], FALSE)
-      }
-    }
-
-    objCOTAN@metaGenes <- setColumnInDF(objCOTAN@metaGenes, lambda,
-                                        "lambda", getGenes(objCOTAN))
+    objCOTAN <- setLambda(objCOTAN, lambda = lambda)
 
     return(objCOTAN)
   }
 )
 
 
+### ------ estimateNuLinear -----
+
 #' @aliases estimateNuLinear
 #'
-#' @details `estimateNuLinear()` does a linear estimation of nu (normalized
+#' @details `estimateNuLinear()` does a linear estimation of `nu` (normalized
 #'   cells' counts averages)
 #'
 #' @param objCOTAN a `COTAN` object
@@ -91,19 +84,7 @@ setMethod(
     nu <- colSums(getRawData(objCOTAN), dims = 1L)
     nu <- nu / mean(nu)
 
-    if (TRUE) {
-      oldNu <- getMetadataCells(objCOTAN)[["nu"]]
-      if (!identical(nu, oldNu)) {
-        # flag the coex slots are out of sync (if any)!
-        objCOTAN@metaDataset <- updateMetaInfo(objCOTAN@metaDataset,
-                                               datasetTags()[["gsync"]], FALSE)
-        objCOTAN@metaDataset <- updateMetaInfo(objCOTAN@metaDataset,
-                                               datasetTags()[["csync"]], FALSE)
-      }
-    }
-
-    objCOTAN@metaCells <- setColumnInDF(objCOTAN@metaCells, nu,
-                                        "nu", getCells(objCOTAN))
+    objCOTAN <- setNu(objCOTAN, nu)
 
     return(objCOTAN)
   }
@@ -134,9 +115,12 @@ setMethod(
 #' @name HandlingClusterizations
 NULL
 
+
+### ------ estimateNuLinearByCluster -----
+
 #' @aliases estimateNuLinearByCluster
 #'
-#' @details `estimateNuLinearByCluster()` does a linear estimation of nu:
+#' @details `estimateNuLinearByCluster()` does a linear estimation of `nu`:
 #'   cells' counts averages normalized *cluster* by *cluster*
 #'
 #' @param objCOTAN a `COTAN` object
@@ -173,73 +157,101 @@ setMethod(
       nu[c] <- nu[c] / mean(nu[c])
     }
 
-    if (TRUE) {
-      oldNu <- getMetadataCells(objCOTAN)[["nu"]]
-      if (!identical(nu, oldNu)) {
-        # flag the coex slots are out of sync (if any)!
-        objCOTAN@metaDataset <- updateMetaInfo(objCOTAN@metaDataset,
-                                               datasetTags()[["gsync"]], FALSE)
-        objCOTAN@metaDataset <- updateMetaInfo(objCOTAN@metaDataset,
-                                               datasetTags()[["csync"]], FALSE)
-      }
-    }
-
-    objCOTAN@metaCells <- setColumnInDF(objCOTAN@metaCells, nu,
-                                        "nu", getCells(objCOTAN))
+    objCOTAN <- setNu(objCOTAN, nu)
 
     return(objCOTAN)
   }
 )
 
+
 # local utility wrapper for parallel estimation of dispersion
 runDispSolver <- function(genesBatches, sumZeros, lambda, nu,
                           threshold, maxIterations, cores) {
+
+  ## forward caller that allows for logging progress
+  worker <- function(genesBatch, sumZeros, lambda, nu,
+                     threshold, maxIterations) {
+    tryCatch({
+      return(parallelDispersionNewton(genes         = genesBatch,
+                                      sumZeros      = sumZeros,
+                                      lambda        = lambda,
+                                      nu            = nu,
+                                      threshold     = threshold,
+                                      maxIterations = maxIterations))
+    }, error = function(e) {
+      structure(list(e), class = "try-error")
+    })
+  }
+
   if (cores != 1L) {
+    ## create a tiny private env `mini` and a worker within it:
+    ## this reduces spawning memory foot-print
+    mini <- new.env(parent = baseenv())
+    mini$parallelDispersionNewton <- parallelDispersionNewton
+    mini$rowsums <- Rfast::rowsums
+    mini$funProbZero <- funProbZero
+    mini$logThis <- logThis
+    mini$worker <- worker
+
+    ## set on the copy in `mini` to only use `mini`
+    environment(mini$parallelDispersionNewton) <- mini
+    environment(mini$worker) <- mini
+
     res <- parallel::mclapply(
       genesBatches,
-      parallelDispersionBisection,
+      worker,
       sumZeros = sumZeros,
       lambda = lambda,
       nu = nu,
       threshold = threshold,
       maxIterations = maxIterations,
-      mc.cores = cores)
+      mc.cores       = cores,
+      mc.preschedule = TRUE, # default
+      mc.cleanup     = TRUE, # default
+      mc.set.seed    = TRUE) # default
+
     # spawned errors are stored as try-error classes
     resError <- unlist(lapply(res, inherits, "try-error"))
     if (any(resError)) {
       stop(res[[which(resError)[[1L]]]], call. = FALSE)
     }
+
     return(res)
   } else {
-    return(lapply(
-      genesBatches,
-      parallelDispersionBisection,
-      sumZeros = sumZeros,
-      lambda = lambda,
-      nu = nu,
-      threshold = threshold,
-      maxIterations = maxIterations))
+    res <- lapply(genesBatches,
+                  parallelDispersionNewton,
+                  sumZeros = sumZeros,
+                  lambda = lambda,
+                  nu = nu,
+                  threshold = threshold,
+                  maxIterations = maxIterations)
+
+    return(res)
   }
 }
 
 
+### ------ estimateDispersionViaSolver -----
+
+#' @aliases estimateDispersionViaSolver
 #' @aliases estimateDispersionBisection
 #'
-#' @details `estimateDispersionBisection()` estimates the negative binomial
-#'   dispersion factor for each gene (a). Determines the `dispersion` such that,
-#'   for each gene, the probability of zero count matches the number of observed
-#'   zeros. It assumes [estimateNuLinear()] being already run.
+#' @details `estimateDispersionViaSolver()` estimates the negative binomial
+#'   dispersion factor for each gene (`dispersion`). Determines the value such
+#'   that, for each gene, the probability of zero count matches the number of
+#'   observed zeros. It assumes [estimateNuLinear()] being already run.
 #'
 #' @param objCOTAN a `COTAN` object
 #' @param threshold minimal solution precision
 #' @param cores number of cores to use. Default is 1.
 #' @param maxIterations max number of iterations (avoids infinite loops)
-#' @param chunkSize number of genes to solve in batch in a single core. Default
-#'   is 1024.
+#' @param chunkSize number of elements to solve in batch in a single core.
+#'   Default is 1024.
 #'
-#' @returns `estimateDispersionBisection()` returns the updated `COTAN` object
+#' @returns `estimateDispersionViaSolver()` returns the updated `COTAN` object
 #'
 #' @importFrom rlang is_null
+#' @importFrom rlang is_empty
 #'
 #' @importFrom assertthat assert_that
 #'
@@ -255,25 +267,32 @@ runDispSolver <- function(genesBatches, sumZeros, lambda, nu,
 #' @export
 #'
 #' @examples
-#' objCOTAN <- estimateDispersionBisection(objCOTAN, cores = 6L)
+#' objCOTAN <- estimateDispersionViaSolver(objCOTAN, cores = 6L)
 #' dispersion <- getDispersion(objCOTAN)
 #'
 #' @rdname ParametersEstimations
 #'
 setMethod(
-  "estimateDispersionBisection",
+  "estimateDispersionViaSolver",
   "COTAN",
   function(objCOTAN, threshold = 0.001, cores = 1L,
            maxIterations = 100L, chunkSize = 1024L) {
-    logThis("Estimate dispersion: START", logLevel = 2L)
+    startTime <- Sys.time()
+
+    logThis("Estimate `dispersion`: START", logLevel = 2L)
 
     cores <- handleMultiCore(cores)
 
     genes <- getGenes(objCOTAN)
     sumZeros <- getNumCells(objCOTAN) - getNumOfExpressingCells(objCOTAN)[genes]
 
-    lambda <- getLambda(objCOTAN)
-    nu <- getNu(objCOTAN)
+    lambda <- suppressWarnings(getLambda(objCOTAN))
+    assert_that(!is_empty(lambda),
+                msg = "`lambda` must not be empty, estimate it")
+
+    nu <- suppressWarnings(getNu(objCOTAN))
+    assert_that(!is_empty(nu),
+                msg = "`nu` must not be empty, estimate it")
 
     dispList <- list()
 
@@ -282,67 +301,37 @@ setMethod(
 
     spGenes <- lapply(spIdx, function(x) genes[x])
 
-    numSplits <- length(spGenes)
-    splitStep <- max(4L, cores * 2L)
+    cores <- min(cores, length(spGenes))
+
+    logThis(paste0("Executing ", length(spGenes), " genes batches"),
+            logLevel = 3L)
+
+    dispList <- runDispSolver(spGenes,
+                              sumZeros      = sumZeros,
+                              lambda        = lambda,
+                              nu            = nu,
+                              threshold     = threshold,
+                              maxIterations = maxIterations,
+                              cores         = cores)
 
     gc()
 
-    pBegin <- 1L
-    while (pBegin <= numSplits) {
-      pEnd <- min(pBegin + splitStep - 1L, numSplits)
+    endTime <- Sys.time()
 
-      logThis(paste0("Executing ", (pEnd - pBegin + 1L), " genes batches from",
-                     " [", spIdx[pBegin], "] to [", spIdx[pEnd], "]"),
-              logLevel = 3L)
+    logThis(paste("Total calculations elapsed time:",
+                  difftime(endTime, startTime, units = "secs")),
+            logLevel = 2L)
 
-      # as the runSolver() might trow, we force up to 5 reruns
-      res <- NULL
-      resError <- "No errors"
-      failCount <- 0L
-      while (!is_null(resError) && failCount < 5L) {
-        failCount <- failCount + 1L
-        c(res, resError) %<-%
-          tryCatch(list(runDispSolver(genesBatches = spGenes[pBegin:pEnd],
-                                      sumZeros = sumZeros,
-                                      lambda = lambda,
-                                      nu = nu,
-                                      threshold = threshold,
-                                      maxIterations = maxIterations,
-                                      cores = cores), NULL),
-                   error = function(e) {
-                     logThis(paste("In genes batches -", e), logLevel = 2L)
-                     list(NULL, e) })
-      }
-
-      assert_that(is_null(resError),
-                  msg = paste("Genes batches failed", failCount,
-                              "times with", resError))
-      dispList <- append(dispList, res)
-      rm(res)
-
-      pBegin <- pEnd + 1L
-    }
-    logThis("Estimate dispersion: DONE", logLevel = 2L)
-
-    gc()
+    logThis("Estimate `dispersion`: DONE", logLevel = 2L)
 
     dispersion <- unlist(dispList, recursive = TRUE, use.names = FALSE)
-    if (TRUE) {
-      oldDispersion <-  getMetadataGenes(objCOTAN)[["dispersion"]]
-      if (!identical(dispersion, oldDispersion)) {
-        # flag the coex slots are out of sync (if any)!
-        objCOTAN@metaDataset <- updateMetaInfo(objCOTAN@metaDataset,
-                                               datasetTags()[["gsync"]], FALSE)
-        objCOTAN@metaDataset <- updateMetaInfo(objCOTAN@metaDataset,
-                                               datasetTags()[["csync"]], FALSE)
-      }
-    }
 
-    objCOTAN@metaGenes <- setColumnInDF(objCOTAN@metaGenes, dispersion,
-                                        "dispersion", genes)
+    objCOTAN <- setDispersion(objCOTAN, dispersion = dispersion)
+
+    logThis("Estimate `dispersion`: DONE", logLevel = 2L)
 
     goodPos <- is.finite(getDispersion(objCOTAN))
-    logThis(paste("dispersion",
+    logThis(paste("`dispersion`",
                   "| min:", min(getDispersion(objCOTAN)[goodPos]),
                   "| max:", max(getDispersion(objCOTAN)[goodPos]),
                   "| % negative:", (sum(getDispersion(objCOTAN) < 0.0) * 100.0 /
@@ -353,10 +342,54 @@ setMethod(
   }
 )
 
+# backward compatibility alias
+estimateDispersionBisection <-
+  function(objCOTAN, threshold = 0.001, cores = 1L,
+           maxIterations = 100L, chunkSize = 1024L) {
+    return(estimateDispersionViaSolver(objCOTAN = objCOTAN,
+                                       threshold = threshold,
+                                       cores = cores,
+                                       maxIterations = maxIterations,
+                                       chunkSize = chunkSize))
+  }
+
+
+
 # local utility wrapper for parallel estimation of nu
 runNuSolver <- function(cellsBatches, sumZeros, lambda, dispersion,
                         initialGuess, threshold, maxIterations, cores) {
+
+  ## forward caller that allows for logging progress
+  worker <- function(batch, sumZeros, lambda, dispersion,
+                     initialGuess, threshold, maxIterations) {
+    tryCatch({
+      return(parallelNuBisection(batch,
+                                 sumZeros      = sumZeros,
+                                 lambda        = lambda,
+                                 dispersion    = dispersion,
+                                 initialGuess  = initialGuess,
+                                 threshold     = threshold,
+                                 maxIterations = maxIterations))
+    }, error = function(e) {
+      structure(list(e), class = "try-error")
+    })
+  }
+
   if (cores != 1L) {
+    # create a tiny private env and a worker within:
+    # it reduces spawning memory foot-print
+    mini <- new.env(parent = baseenv())
+    mini$parallelNuBisection <- parallelNuBisection
+    mini$assert_that <- assertthat::assert_that
+    mini$colsums <- Rfast::colsums
+    mini$funProbZero <- funProbZero
+    mini$logThis <- logThis
+    mini$worker <- worker
+
+    ## set on the copy in `mini` to only use `mini`
+    environment(mini$parallelNuBisection) <- mini
+    environment(mini$worker) <- mini
+
     res <- parallel::mclapply(
       cellsBatches,
       parallelNuBisection,
@@ -366,32 +399,41 @@ runNuSolver <- function(cellsBatches, sumZeros, lambda, dispersion,
       initialGuess = initialGuess,
       threshold = threshold,
       maxIterations = maxIterations,
-      mc.cores = cores)
+      mc.cores       = cores,
+      mc.preschedule = TRUE, # default
+      mc.cleanup     = TRUE, # default
+      mc.set.seed    = TRUE) # default
+
     # spawned errors are stored as try-error classes
     resError <- unlist(lapply(res, inherits, "try-error"))
     if (any(resError)) {
       stop(res[[which(resError)[[1L]]]], call. = FALSE)
     }
+
     return(res)
   } else {
-    return(lapply(
-      cellsBatches,
-      parallelNuBisection,
-      sumZeros = sumZeros,
-      lambda = lambda,
-      dispersion = dispersion,
-      initialGuess = initialGuess,
-      threshold = threshold,
-      maxIterations = maxIterations))
+    res <- lapply(cellsBatches,
+                  parallelNuBisection,
+                  sumZeros = sumZeros,
+                  lambda = lambda,
+                  dispersion = dispersion,
+                  initialGuess = initialGuess,
+                  threshold = threshold,
+                  maxIterations = maxIterations)
+
+    return(res)
   }
 }
 
+
+### ------ estimateNuBisection -----
+
 #' @aliases estimateNuBisection
 #'
 #' @details `estimateNuBisection()` estimates the `nu` vector of a `COTAN`
 #'   object by bisection. It determines the `nu` parameters such that, for each
 #'   cell, the probability of zero counts matches the number of observed zeros.
-#'   It assumes [estimateDispersionBisection()] being already run. Since this
+#'   It assumes [estimateDispersionViaSolver()] being already run. Since this
 #'   breaks the assumption that the average `nu` is one, it is recommended not
 #'   to run this in isolation but use [estimateDispersionNuBisection()] instead.
 #'
@@ -399,13 +441,13 @@ runNuSolver <- function(cellsBatches, sumZeros, lambda, dispersion,
 #' @param threshold minimal solution precision
 #' @param cores number of cores to use. Default is 1.
 #' @param maxIterations max number of iterations (avoids infinite loops)
-#' @param chunkSize number of genes to solve in batch in a single core. Default
-#'   is 1024.
+#' @param chunkSize number of elements to solve in batch in a single core.
+#'   Default is 1024.
 #'
 #' @returns `estimateNuBisection()` returns the updated `COTAN` object
 #'
-#' @importFrom rlang is_empty
 #' @importFrom rlang is_null
+#' @importFrom rlang is_empty
 #'
 #' @importFrom assertthat assert_that
 #'
@@ -427,25 +469,30 @@ setMethod(
   "COTAN",
   function(objCOTAN, threshold = 0.001, cores = 1L,
            maxIterations = 100L, chunkSize = 1024L) {
-    logThis("Estimate nu: START", logLevel = 2L)
+    startTime <- Sys.time()
+
+    logThis("Estimate `nu`: START", logLevel = 2L)
 
     cores <- handleMultiCore(cores)
 
     # parameters estimation
-    if (is_empty(getNu(objCOTAN))) {
-      stop("nu must not be empty, estimate it")
-    }
-
-    if (is_empty(getDispersion(objCOTAN))) {
-      stop("dispersion must not be empty, estimate it")
-    }
 
     cells <- getCells(objCOTAN)
     sumZeros <- getNumGenes(objCOTAN) - getNumExpressedGenes(objCOTAN)
 
-    lambda <- getLambda(objCOTAN)
-    dispersion <- getDispersion(objCOTAN)
-    initialGuess <- getNu(objCOTAN)
+    lambda <- suppressWarnings(getLambda(objCOTAN))
+    assert_that(!is_empty(lambda),
+                msg = "`lambda` must not be empty, estimate it")
+
+    nu <- suppressWarnings(getNu(objCOTAN))
+    assert_that(!is_empty(nu),
+                msg = "`nu` must not be empty, estimate it")
+
+    dispersion <- suppressWarnings(getDispersion(objCOTAN))
+    assert_that(!is_empty(dispersion),
+                msg = "`dispersion` must not be empty, estimate it")
+
+    initialGuess <- nu
 
     nuList <- list()
 
@@ -454,66 +501,37 @@ setMethod(
 
     spCells <- lapply(spIdx, function(x) cells[x])
 
-    numSplits <- length(spCells)
-    splitStep <- max(4L, cores * 2L)
+    cores <- min(cores, length(spCells))
+
+    logThis(paste0("Executing ", length(spCells), " cells batches"),
+            logLevel = 3L)
+
+    nuList <- runNuSolver(spCells,
+                          sumZeros = sumZeros,
+                          lambda = lambda,
+                          dispersion = dispersion,
+                          initialGuess = initialGuess,
+                          threshold = threshold,
+                          maxIterations = maxIterations,
+                          cores = cores)
 
     gc()
 
-    pBegin <- 1L
-    while (pBegin <= numSplits) {
-      pEnd <- min(pBegin + splitStep - 1L, numSplits)
+    endTime <- Sys.time()
 
-      logThis(paste0("Executing ", (pEnd - pBegin + 1L), " cells batches from",
-                     " [", spIdx[pBegin], "] to [", spIdx[pEnd], "]"),
-              logLevel = 3L)
+    logThis(paste("Total calculations elapsed time:",
+                  difftime(endTime, startTime, units = "secs")),
+            logLevel = 2L)
 
-      # as the runSolver() might trow, we force up to 5 reruns
-      res <- NULL
-      resError <- "No errors"
-      failCount <- 0L
-      while (!is_null(resError) && failCount < 5L) {
-        failCount <- failCount + 1L
-        c(res, resError) %<-%
-          tryCatch(list(runNuSolver(spCells[pBegin:pEnd],
-                                    sumZeros = sumZeros,
-                                    lambda = lambda,
-                                    dispersion = dispersion,
-                                    initialGuess = initialGuess,
-                                    threshold = threshold,
-                                    maxIterations = maxIterations,
-                                    cores = cores), NULL),
-                   error = function(e) {
-                     logThis(paste("In cells batches -", e), logLevel = 2L)
-                     list(NULL, e) })
-      }
-
-      assert_that(is_null(resError),
-                  msg = paste("Cells batches failed", failCount,
-                              "times with", resError))
-      nuList <- append(nuList, res)
-      rm(res)
-
-      pBegin <- pEnd + 1L
-    }
-
-    gc()
-    logThis("Estimate nu: DONE", logLevel = 2L)
+    logThis("Estimate `nu`: DONE", logLevel = 2L)
 
     nu <- unlist(nuList, recursive = TRUE, use.names = FALSE)
-    if (!identical(nu, initialGuess)) {
-      # flag the coex slots are out of sync (if any)!
-      objCOTAN@metaDataset <- updateMetaInfo(objCOTAN@metaDataset,
-                                             datasetTags()[["gsync"]], FALSE)
-      objCOTAN@metaDataset <- updateMetaInfo(objCOTAN@metaDataset,
-                                             datasetTags()[["csync"]], FALSE)
-    }
 
-    objCOTAN@metaCells <- setColumnInDF(objCOTAN@metaCells, nu,
-                                        "nu", cells)
+    objCOTAN <- setNu(objCOTAN, nu = nu)
 
     if (TRUE) {
       nuChange <- abs(nu - initialGuess)
-      logThis(paste("nu change (abs)",
+      logThis(paste("`nu` change (abs)",
                     "| max:",     max(nuChange),
                     "| median: ", median(nuChange),
                     "| mean: ",   mean(nuChange)),
@@ -525,6 +543,8 @@ setMethod(
 )
 
 
+### ------ estimateDispersionNuBisection -----
+
 #' @aliases estimateDispersionNuBisection
 #'
 #' @details `estimateDispersionNuBisection()` estimates the `dispersion` and
@@ -535,8 +555,8 @@ setMethod(
 #' @param threshold minimal solution precision
 #' @param cores number of cores to use. Default is 1.
 #' @param maxIterations max number of iterations (avoids infinite loops)
-#' @param chunkSize number of genes to solve in batch in a single core. Default
-#'   is 1024.
+#' @param chunkSize number of elements to solve in batch in a single core.
+#'   Default is 1024.
 #' @param enforceNuAverageToOne a Boolean on whether to keep the average `nu`
 #'   equal to 1
 #'
@@ -564,10 +584,12 @@ setMethod(
   function(objCOTAN, threshold = 0.001, cores = 1L,
            maxIterations = 100L, chunkSize = 1024L,
            enforceNuAverageToOne = TRUE) {
-    logThis("Estimate 'dispersion'/'nu': START", logLevel = 2L)
+    startTime <- Sys.time()
 
-    # getNu() would show a warning when no 'nu' present
-    if (is_empty(getMetadataCells(objCOTAN)[["nu"]])) {
+    logThis("Estimate `dispersion`/`nu`: START", logLevel = 2L)
+
+    # getNu() would show a warning when no `nu` present
+    if (is_empty(suppressWarnings(getNu(objCOTAN)))) {
       objCOTAN <- estimateNuLinear(objCOTAN)
     }
 
@@ -575,8 +597,8 @@ setMethod(
 
     iter <- 1L
     repeat {
-      # a smalle threshold is used in order to ensure the global convergence
-      objCOTAN <- estimateDispersionBisection(objCOTAN,
+      # a smaller threshold is used in order to ensure the global convergence
+      objCOTAN <- estimateDispersionViaSolver(objCOTAN,
                                               threshold = threshold / 10.0,
                                               cores = cores,
                                               maxIterations = maxIterations,
@@ -593,7 +615,7 @@ setMethod(
       gc()
 
       meanNu <- mean(getNu(objCOTAN))
-      logThis(paste("Nu mean:", meanNu), logLevel = 2L)
+      logThis(paste("`nu` mean:", meanNu), logLevel = 2L)
 
       if (isTRUE(enforceNuAverageToOne)) {
         if (is.finite(meanNu)) {
@@ -601,9 +623,9 @@ setMethod(
             setColumnInDF(objCOTAN@metaCells, getNu(objCOTAN) / meanNu, "nu")
         } else {
           assert_that(iter == 1L,
-                      msg = paste("It can happen to have infinite mean 'nu'",
+                      msg = paste("It can happen to have infinite mean `nu`",
                                   "only after the first loop"))
-          warning("Infinite 'nu' found: one of the cells expressed all genes\n",
+          warning("Infinite `nu` found: one of the cells expressed all genes\n",
                   " Setting 'enforceNuAverageToOne <- FALSE'")
           enforceNuAverageToOne <- FALSE
         }
@@ -634,12 +656,21 @@ setMethod(
       iter <- iter + 1L
     }
 
-    logThis("Estimate dispersion/nu: DONE", logLevel = 2L)
+    endTime <- Sys.time()
+
+    logThis(paste("Total calculations elapsed time:",
+                  difftime(endTime, startTime, units = "secs")),
+            logLevel = 2L)
+
+    logThis("Estimate `dispersion`/`nu`: DONE", logLevel = 2L)
 
     return(objCOTAN)
   }
 )
 
+
+### ------ estimateDispersionNuNlminb -----
+
 #' @aliases estimateDispersionNuNlminb
 #'
 #' @details `estimateDispersionNuNlminb()` estimates the `nu` and
@@ -651,22 +682,23 @@ setMethod(
 #' @param objCOTAN a `COTAN` object
 #' @param threshold minimal solution precision
 #' @param maxIterations max number of iterations (avoids infinite loops)
-#' @param chunkSize number of genes to solve in batch in a single core. Default
-#'   is 1024.
+#' @param chunkSize number of elements to solve in batch in a single core.
+#'   Default is 1024.
 #' @param enforceNuAverageToOne a Boolean on whether to keep the average `nu`
 #'   equal to 1
 #'
 #' @returns `estimateDispersionNuNlminb()` returns the updated `COTAN` object
 #'
+#' @importFrom rlang is_null
 #' @importFrom rlang is_empty
 #'
+#' @importFrom assertthat assert_that
+#'
 #' @importFrom Rfast rowsums
 #' @importFrom Rfast colsums
 #'
 #' @importFrom stats nlminb
 #'
-#' @importFrom assertthat assert_that
-#'
 #' @rdname ParametersEstimations
 #'
 setMethod(
@@ -675,15 +707,20 @@ setMethod(
   function(objCOTAN, threshold = 0.001,
            maxIterations = 50L, chunkSize = 1024L,
            enforceNuAverageToOne = TRUE) {
-    logThis("Estimate 'dispersion'/'nu': START", logLevel = 2L)
+    startTime <- Sys.time()
 
-    # getNu() would show a warning when no 'nu' present
-    if (is_empty(getMetadataCells(objCOTAN)[["nu"]])) {
+    logThis("Estimate `dispersion`/`nu`: START", logLevel = 2L)
+
+    lambda <- suppressWarnings(getLambda(objCOTAN))
+    assert_that(!is_empty(lambda),
+                msg = "`lambda` must not be empty, estimate it")
+
+    nu <- suppressWarnings(getNu(objCOTAN))
+    if (is_empty(nu)) {
       objCOTAN <- estimateNuLinear(objCOTAN)
+      nu <- getNu(objCOTAN)
     }
 
-    lambda <- getLambda(objCOTAN)
-
     zeroGenes <- getNumCells(objCOTAN) - getNumOfExpressingCells(objCOTAN)
     zeroCells <- getNumGenes(objCOTAN) - getNumExpressedGenes(objCOTAN)
 
@@ -725,13 +762,16 @@ setMethod(
     dispersion <- solution[["par"]][1L:numGenes]
     nu <- exp(solution[["par"]][(numGenes + 1L):length(solution[["par"]])])
 
-    objCOTAN@metaGenes <- setColumnInDF(objCOTAN@metaGenes, dispersion,
-                                        "dispersion", getGenes(objCOTAN))
+    objCOTAN <- setDispersion(objCOTAN, dispersion = dispersion)
+    objCOTAN <- setNu(objCOTAN, nu = nu)
 
-    objCOTAN@metaCells <- setColumnInDF(objCOTAN@metaCells, nu,
-                                        "nu", getCells(objCOTAN))
+    endTime <- Sys.time()
 
-    logThis("Estimate 'dispersion'/'nu': DONE", logLevel = 2L)
+    logThis(paste("Total calculations elapsed time:",
+                  difftime(endTime, startTime, units = "secs")),
+            logLevel = 2L)
+
+    logThis("Estimate `dispersion`/`nu`: DONE", logLevel = 2L)
 
     return(objCOTAN)
   }
